<?php declare (strict_types = 1);

namespace EuKit\Base\ViewModels;

use EuKit\Base\Parts\Image;
use EuKit\Base\Parts\NewsMLArticle;
use Infomaker\Everyware\NewsML\Objects\TeaserItem;
use Infomaker\Everyware\Support\GenericPropertyObject;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;

/**
 * Class TeaserArticle
 * @package EuKit\Base
 */
class Teaser extends BaseObject
{
    /**
     * @var PropertyObject
     */
    private $article;

    /**
     * @var string|null
     */
    private $imageRatio;

    public function __construct(PropertyObject $article)
    {
        $this->article = $article;
    }

    /**
     * @param string|null $imageRatio Examples: '16:9', '3:2'. Null reverts it to the default image ratio.
     */
    public function setImageRatio(?string $imageRatio): void
    {
        $this->imageRatio = $imageRatio;
    }

    private function createTeaser(): void
    {
        $propertyObject = new GenericPropertyObject();
        $propertyObject->fill([
            'uuid' => $this->article->getSingleValue('uuid'),
            'pubdate' => $this->article->getPubDate()->diffForHumans(),
            'pubdatetime' => $this->article->getPubDate(),
            'headline' => $this->article->getSingleValue('headline'),
            'leadin' => $this->article->getSingleValue('leadin'),
            'authors' => $this->article->authors,
            'permalink' => $this->article->getSingleValue('permalink'),
            'sectionName' => $this->article->getSingleValue('section'),
            'categoriesName' => $this->article->categories->first() ? $this->article->categories->first()->getFirstOfSingleValue('name') : '',
            'tagsName' => $this->article->tags->first() ? $this->article->tags->first()->getFirstOfSingleValue('name') : '',
            'relatedArticles' => $this->article->articles,
        ]);

        $teaser = $this->article->getNewsMLTeaser();
        if ($teaser instanceof TeaserItem) {
            $image = $teaser->hasImage() ? new Image($teaser->getImage()) : null;
            $text = $teaser->getItemAttribute('text');

            if ($this->article instanceof NewsMLArticle) {
                $bodyParser = $this->article->getBodyParser();

                if ($image === null) {
                    $imageType = 'x-im/image';
                    $image = $bodyParser->hasType($imageType) ? new Image($bodyParser->firstOfType($imageType)) : null;
                }

                if($text === "") {
                    $leadinType = "preamble";
                    $text = $bodyParser->hasType($leadinType) ? $bodyParser->firstOfType($leadinType)->get('content') : "";
                }
            }

            if ($image instanceof Image && $this->imageRatio !== null) {
                $image->setRatio($this->imageRatio);
            }

            $teaserData = array_replace($teaser->toArray(), [
                'image' => $image instanceof Image ? $image->getViewData() : null,
                'text' => $text,
            ]);

            $propertyObject->fill(array_filter($teaserData));
        }

        parent::__construct($propertyObject);
    }

    public function getViewData(): array {
        if ($this->viewData === null) {
            $this->createTeaser();
        }

        return parent::getViewData();
    }
}
